﻿using ComputersShopContracts.BindingModels;
using ComputersShopContracts.SearchModels;
using ComputersShopContracts.StoragesContracts;
using ComputersShopContracts.ViewModels;
using ComputersShopDataModels.Models;
using ComputersShopFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputersShopFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton source;

        public ShopStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop != null)
            {
                source.Shops.Remove(shop);
                source.SaveShops();
                return shop.GetViewModel;
            }
            return null;
        }

        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }
            return source.Shops
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if(string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            return source.Shops
                .Where(x => x.ShopName.Contains(model.ShopName))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<ShopViewModel> GetFullList()
        {
            return source.Shops.Select(x => x.GetViewModel).ToList();
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = source.Shops.Count > 0 ? source.Shops.Max(x => x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            source.Shops.Add(newShop);
            source.SaveShops();
            return newShop.GetViewModel;
        }

        public bool SellComputers(IComputerModel model, int quantity)
        {
            int availableQuantity = source.Shops.Select(x => x.Computers.FirstOrDefault(y => y.Key == model.Id).Value.Item2).Sum();
            if (availableQuantity < quantity)
            {
                return false;
            }
            var shops = source.Shops.Where(x => x.Computers.ContainsKey(model.Id));
            foreach (var shop in shops)
            {
                int countInCurrentShop = shop.Computers[model.Id].Item2;
                if (countInCurrentShop <= quantity)
                {
                    shop.Computers[model.Id] = (shop.Computers[model.Id].Item1, 0);
                    quantity -= countInCurrentShop;
                }
                else
                {
                    shop.Computers[model.Id] = (shop.Computers[model.Id].Item1, countInCurrentShop - quantity);
                    quantity = 0;
                }
                Update(new ShopBindingModel
                {
                    Id = shop.Id,
                    ShopName = shop.ShopName,
                    ShopAddress = shop.ShopAddress,
                    DateOpening = shop.DateOpening,
                    Computers = shop.Computers,
                    Capacity = shop.Capacity
                });
                if (quantity == 0)
                {
                    return true;
                }
            }
            return false;
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }
            shop.Update(model);
            source.SaveShops();
            return shop.GetViewModel;
        }
    }
}
